package org.apache.myfaces.tobago.example.addressbook;

/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * Created 29.11.2004 17:36:20.
 * $Id: Controller.java,v 1.2 2005/08/10 11:57:55 lofwyr Exp $
 */

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

public class InMemoryAddressDAO implements AddressDAO {

  private static final Log LOG = LogFactory.getLog(InMemoryAddressDAO.class);

  private List<Address> addresses;

  public InMemoryAddressDAO() {
    addresses = new ArrayList<Address>();
  }

  public synchronized Address updateAddress(Address address) {
    LOG.debug("Trying address: "+address);
    Address storedAddress = getAddress(address.getId());
    if (storedAddress == null) {
      address.setId(addresses.size()+1);
      LOG.debug("Creating address: "+address);
      addresses.add(address);
    } else {
      LOG.debug("Updating address : "+address);
      LOG.debug("Stored address is: "+storedAddress);
      storedAddress.fill(address);
    }
    return address;
  }

  public synchronized List<Address> findAddresses() {
    LOG.debug("Find addresses: "+addresses);
    return Collections.unmodifiableList(addresses);
  }

  public synchronized void removeAddress(Address address) {
    Iterator<Address> it = addresses.iterator();
    while (it.hasNext()) {
      if (it.next().getId() == address.getId()) {
        it.remove();
      }
    }
  }

  private Address getAddress(int id) {
    for (Address address : addresses) {
      if (address.getId() == id) {
        return address;
      }
    }
    return null;
  }
}
